@@ -6,7 +6,6 @@ from abc import abstractmethod
 from typing import Callable, Optional, Union
 import logging
 
-from time import time
 import numpy as np
 from scipy import sparse  # import sparse.coo_matrix, sparse.bmat, sparse.eye
 from ..interpolators import InterpolatorType
@@ -33,6 +32,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
         GeologicalInterpolator.__init__(self, data=data, up_to_date=up_to_date)
         self.B = []
         self.support = support
+        self.dimensions = support.dimension
         self.c = (
             np.array(c)
             if c is not None and np.array(c).shape[0] == self.support.n_nodes
@@ -42,7 +42,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
 
         self.shape = "rectangular"
         if self.shape == "square":
-            self.B = np.zeros(self.nx)
+            self.B = np.zeros(self.dof)
         self.c_ = 0
 
         self.solver = "cg"
@@ -60,12 +60,25 @@ class DiscreteInterpolator(GeologicalInterpolator):
         self.non_linear_constraints = []
         self.constraints = {}
         self.interpolation_weights = {}
-        logger.info("Creating discrete interpolator with {} degrees of freedom".format(self.nx))
+        logger.info("Creating discrete interpolator with {} degrees of freedom".format(self.dof))
         self.type = InterpolatorType.BASE_DISCRETE
-        self.c = np.zeros(self.support.n_nodes)
+
+    def set_nelements(self, nelements: int) -> int:
+        return self.support.set_nelements(nelements)
 
     @property
-    def nx(self) -> int:
+    def n_elements(self) -> int:
+        """Number of elements in the interpolator
+
+        Returns
+        -------
+        int
+            number of elements, positive
+        """
+        return self.support.n_elements
+
+    @property
+    def dof(self) -> int:
         """Number of degrees of freedom for the interpolator
 
         Returns
@@ -112,7 +125,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
         # self.region_function = region
         logger.info(
             "Cannot use region at the moment. Interpolation now uses region and has {} degrees of freedom".format(
-                self.nx
+                self.dof
             )
         )
 
@@ -133,6 +146,28 @@ class DiscreteInterpolator(GeologicalInterpolator):
             self.up_to_date = False
             self.interpolation_weights[key] = weights[key]
 
+    def _pre_solve(self):
+        """
+        Pre solve function to be run before solving the interpolation
+        """
+        self.c = np.zeros(self.support.n_nodes)
+        self.c[:] = np.nan
+        return True
+
+    def _post_solve(self):
+        """Post solve function(s) to be run after the solver has been called"""
+        self.clear_constraints()
+        return True
+
+    def clear_constraints(self):
+        """
+        Clear the constraints from the interpolator, this makes sure we are not storing
+        the constraints after the solver has been run
+        """
+        self.constraints = {}
+        self.ineq_constraints = {}
+        self.equal_constraints = {}
+
     def reset(self):
         """
         Reset the interpolation constraints
@@ -140,7 +175,8 @@ class DiscreteInterpolator(GeologicalInterpolator):
         """
         self.constraints = {}
         self.c_ = 0
-        logger.debug("Resetting interpolation constraints")
+        self.regularisation_scale = np.ones(self.dof)
+        logger.info("Resetting interpolation constraints")
 
     def add_constraints_to_least_squares(self, A, B, idc, w=1.0, name="undefined"):
         """
@@ -181,7 +217,9 @@ class DiscreteInterpolator(GeologicalInterpolator):
             B = B.reshape((A.shape[0]))
             # w = w.reshape((A.shape[0]))
         # normalise by rows of A
-        length = np.linalg.norm(A, axis=1)  # .getcol(0).norm()
+        # Should this be done? It should make the solution more stable
+        length = np.linalg.norm(A, axis=1)
+        # length[length>0] = 1.
         B[length > 0] /= length[length > 0]
         # going to assume if any are nan they are all nan
         mask = np.any(np.isnan(A), axis=1)
@@ -193,9 +231,6 @@ class DiscreteInterpolator(GeologicalInterpolator):
             raise BaseException("w must be a numpy array")
 
         if w.shape[0] != A.shape[0]:
-            #     # make w the same size as A
-            #     w = np.tile(w,(A.shape[1],1)).T
-            # else:
             raise BaseException("Weight array does not match number of constraints")
         if np.any(np.isnan(idc)) or np.any(np.isnan(A)) or np.any(np.isnan(B)):
             logger.warning("Constraints contain nan not adding constraints: {}".format(name))
@@ -211,7 +246,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
         rows = np.tile(rows, (A.shape[-1], 1)).T
         self.constraints[name] = {
             'matrix': sparse.coo_matrix(
-                (A.flatten(), (rows.flatten(), idc.flatten())), shape=(n_rows, self.nx)
+                (A.flatten(), (rows.flatten(), idc.flatten())), shape=(n_rows, self.dof)
             ).tocsc(),
             'b': B.flatten(),
             'w': w,
@@ -252,7 +287,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
         A : numpy array
             matrix of coefficients
         bounds : numpy array
-            nx3 lower, upper, 1
+            n*3 lower, upper, 1
         idc : numpy array
             index of constraints in the matrix
         Returns
@@ -262,14 +297,14 @@ class DiscreteInterpolator(GeologicalInterpolator):
         # map from mesh node index to region node index
         gi = np.zeros(self.support.n_nodes, dtype=int)
         gi[:] = -1
-        gi[self.region] = np.arange(0, self.nx, dtype=int)
+        gi[self.region] = np.arange(0, self.dof, dtype=int)
         idc = gi[idc]
         rows = np.arange(0, idc.shape[0])
         rows = np.tile(rows, (A.shape[-1], 1)).T
 
         self.ineq_constraints[name] = {
             'matrix': sparse.coo_matrix(
-                (A.flatten(), (rows.flatten(), idc.flatten())), shape=(rows.shape[0], self.nx)
+                (A.flatten(), (rows.flatten(), idc.flatten())), shape=(rows.shape[0], self.dof)
             ).tocsc(),
             "bounds": bounds,
         }
@@ -286,31 +321,43 @@ class DiscreteInterpolator(GeologicalInterpolator):
             self.add_inequality_constraints_to_matrix(a, points[:, 3:5], cols, 'inequality_value')
 
     def add_inequality_pairs_constraints(
-        self, w: float = 1.0, upper_bound=np.finfo(float).eps, lower_bound=-np.inf
+        self,
+        w: float = 1.0,
+        upper_bound=np.finfo(float).eps,
+        lower_bound=-np.inf,
+        pairs: Optional[list] = None,
     ):
 
         points = self.get_inequality_pairs_constraints()
         if points.shape[0] > 0:
-
             # assemble a list of pairs in the model
             # this will make pairs even across stratigraphic boundaries
             # TODO add option to only add stratigraphic pairs
-            pairs = {}
-            k = 0
-            for i in np.unique(points[:, self.support.dimension]):
-                for j in np.unique(points[:, self.support.dimension]):
-                    if i == j:
-                        continue
-                    if tuple(sorted([i, j])) not in pairs:
-                        pairs[tuple(sorted([i, j]))] = k
-                        k += 1
-            pairs = list(pairs.keys())
+            if not pairs:
+                pairs = {}
+                k = 0
+                for i in np.unique(points[:, self.support.dimension]):
+                    for j in np.unique(points[:, self.support.dimension]):
+                        if i == j:
+                            continue
+                        if tuple(sorted([i, j])) not in pairs:
+                            pairs[tuple(sorted([i, j]))] = k
+                            k += 1
+                pairs = list(pairs.keys())
             for pair in pairs:
                 upper_points = points[points[:, self.support.dimension] == pair[0]]
                 lower_points = points[points[:, self.support.dimension] == pair[1]]
 
                 upper_interpolation = self.support.get_element_for_location(upper_points)
                 lower_interpolation = self.support.get_element_for_location(lower_points)
+                if (~upper_interpolation[3]).sum() > 0:
+                    logger.warning(
+                        f'Upper points not in mesh {upper_points[~upper_interpolation[3]]}'
+                    )
+                if (~lower_interpolation[3]).sum() > 0:
+                    logger.warning(
+                        f'Lower points not in mesh {lower_points[~lower_interpolation[3]]}'
+                    )
                 ij = np.array(
                     [
                         *np.meshgrid(
@@ -325,7 +372,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
                 rows = np.arange(0, ij.shape[0], dtype=int)
                 rows = np.tile(rows, (upper_interpolation[1].shape[-1], 1)).T
                 rows = np.hstack([rows, rows])
-                a = upper_interpolation[1][upper_interpolation[3]][ij[:, 0]]  # np.ones(ij.shape[0])
+                a = upper_interpolation[1][upper_interpolation[3]][ij[:, 0]]
                 a = np.hstack([a, -lower_interpolation[1][lower_interpolation[3]][ij[:, 1]]])
                 cols = np.hstack(
                     [
@@ -385,7 +432,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
             np.ones((value.shape[0], 1)),
             l,
             u,
-            np.arange(0, self.nx, dtype=int),
+            np.arange(0, self.dof, dtype=int),
         )
 
     def add_equality_constraints(self, node_idx, values, name="undefined"):
@@ -408,7 +455,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
         # map from mesh node index to region node index
         gi = np.zeros(self.support.n_nodes)
         gi[:] = -1
-        gi[self.region] = np.arange(0, self.nx)
+        gi[self.region] = np.arange(0, self.dof)
         idc = gi[node_idx]
         outside = ~(idc == -1)
 
@@ -469,7 +516,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
         if len(self.equal_constraints) > 0:
             ATA = A.T.dot(A)
             ATB = A.T.dot(B)
-            logger.info(f"Equality block is {self.eq_const_c} x {self.nx}")
+            logger.info(f"Equality block is {self.eq_const_c} x {self.dof}")
             # solving constrained least squares using
             # | ATA CT | |c| = b
             # | C   0  | |y|   d
@@ -494,7 +541,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
 
             C = sparse.coo_matrix(
                 (np.array(a), (np.array(rows), cols)),
-                shape=(self.eq_const_c, self.nx),
+                shape=(self.eq_const_c, self.dof),
                 dtype=float,
             ).tocsr()
 
@@ -511,7 +558,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
             mats.append(c['matrix'])
             bounds.append(c['bounds'])
         if len(mats) == 0:
-            return None, None
+            return sparse.csr_matrix((0, self.dof), dtype=float), np.zeros((0, 3))
         Q = sparse.vstack(mats)
         bounds = np.vstack(bounds)
         return Q, bounds
@@ -519,6 +566,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
     def solve_system(
         self,
         solver: Optional[Union[Callable[[sparse.csr_matrix, np.ndarray], np.ndarray], str]] = None,
+        tol: Optional[float] = None,
         solver_kwargs: dict = {},
     ) -> bool:
         """
@@ -539,19 +587,16 @@ class DiscreteInterpolator(GeologicalInterpolator):
             True if the interpolation is run
 
         """
-        starttime = time()
-        self.c = np.zeros(self.support.n_nodes)
-        self.c[:] = np.nan
+        if not self._pre_solve():
+            raise ValueError("Pre solve failed")
+
         A, b = self.build_matrix()
         Q, bounds = self.build_inequality_matrix()
         if callable(solver):
             logger.warning('Using custom solver')
             self.c = solver(A.tocsr(), b)
             self.up_to_date = True
-
-            return True
-        ## solve with lsmr
-        if isinstance(solver, str):
+        elif isinstance(solver, str) or solver is None:
             if solver not in ['cg', 'lsmr', 'admm']:
                 logger.warning(
                     f'Unknown solver {solver} using cg. \n Available solvers are cg and lsmr or a custom solver as a callable function'
@@ -559,18 +604,29 @@ class DiscreteInterpolator(GeologicalInterpolator):
                 solver = 'cg'
         if solver == 'cg':
             logger.info("Solving using cg")
-            ATA = A.T.dot(A)
-            ATB = A.T.dot(b)
-            res = sparse.linalg.cg(ATA, ATB, **solver_kwargs)
+            if 'atol' not in solver_kwargs or 'rtol' not in solver_kwargs:
+                if tol is not None:
+                    solver_kwargs['atol'] = tol
+
+            logger.info(f"Solver kwargs: {solver_kwargs}")
+
+            res = sparse.linalg.cg(A.T @ A, A.T @ b, **solver_kwargs)
             if res[1] > 0:
                 logger.warning(
                     f'CG reached iteration limit ({res[1]})and did not converge, check input data. Setting solution to last iteration'
                 )
             self.c = res[0]
             self.up_to_date = True
-            return True
+
         elif solver == 'lsmr':
             logger.info("Solving using lsmr")
+            if 'atol' not in solver_kwargs:
+                if tol is not None:
+                    solver_kwargs['atol'] = tol
+            if 'btol' not in solver_kwargs:
+                if tol is not None:
+                    solver_kwargs['btol'] = tol
+            logger.info(f"Solver kwargs: {solver_kwargs}")
             res = sparse.linalg.lsmr(A, b, **solver_kwargs)
             if res[1] == 1 or res[1] == 4 or res[1] == 2 or res[1] == 5:
                 self.c = res[0]
@@ -585,8 +641,7 @@ class DiscreteInterpolator(GeologicalInterpolator):
                 )
                 self.c = res[0]
             self.up_to_date = True
-            logger.info("Interpolation took %f seconds" % (time() - starttime))
-            return True
+
         elif solver == 'admm':
             logger.info("Solving using admm")
 
@@ -601,28 +656,35 @@ class DiscreteInterpolator(GeologicalInterpolator):
 
             try:
                 from loopsolver import admm_solve
+
+                try:
+                    linsys_solver = solver_kwargs.pop('linsys_solver', 'lsmr')
+                    res = admm_solve(
+                        A,
+                        b,
+                        Q,
+                        bounds,
+                        x0=x0,
+                        admm_weight=solver_kwargs.pop('admm_weight', 0.01),
+                        nmajor=solver_kwargs.pop('nmajor', 200),
+                        linsys_solver_kwargs=solver_kwargs,
+                        linsys_solver=linsys_solver,
+                    )
+                    self.c = res
+                    self.up_to_date = True
+                except ValueError as e:
+                    logger.error(f"ADMM solver failed: {e}")
+                    self.up_to_date = False
             except ImportError:
                 logger.warning(
                     "Cannot import admm solver. Please install loopsolver or use lsmr or cg"
                 )
-                return False
-            try:
-                res = admm_solve(
-                    A,
-                    b,
-                    Q,
-                    bounds,
-                    x0=x0,
-                    admm_weight=solver_kwargs.pop('admm_weight', 0.01),
-                    nmajor=solver_kwargs.pop('nmajor', 200),
-                    linsys_solver_kwargs=solver_kwargs,
-                )
-                self.c = res
-                self.up_to_date = True
-            except ValueError as e:
-                logger.error(f"ADMM solver failed: {e}")
-                return False
-        return False
+                self.up_to_date = False
+        else:
+            logger.error(f"Unknown solver {solver}")
+            self.up_to_date = False
+        # self._post_solve()
+        return self.up_to_date
 
     def update(self) -> bool:
         """
@@ -641,7 +703,8 @@ class DiscreteInterpolator(GeologicalInterpolator):
             return False
         if not self.up_to_date:
             self.setup_interpolator()
-            return self.solve_system(self.solver)
+            self.up_to_date = self.solve_system(self.solver)
+            return self.up_to_date
 
     def evaluate_value(self, locations: np.ndarray) -> np.ndarray:
         """Evaluate the value of the interpolator at location
@@ -690,3 +753,6 @@ class DiscreteInterpolator(GeologicalInterpolator):
             **super().to_dict(),
             # 'region_function':self.region_function,
         }
+
+    def vtk(self):
+        return self.support.vtk({'c': self.c})
