@@ -2,11 +2,12 @@
 Main entry point for creating a geological model
 """
 
+from LoopStructural import LoopStructuralConfig
 from ...utils import getLogger
 
 import numpy as np
 import pandas as pd
-from typing import List, Optional
+from typing import List, Optional, Union, Dict
 import pathlib
 from ...modelling.features.fault import FaultSegment
 
@@ -20,6 +21,7 @@ from ...modelling.features import (
     UnconformityFeature,
     StructuralFrame,
     GeologicalFeature,
+    BaseFeature,
     FeatureType,
 )
 from ...modelling.features.fold import (
@@ -123,8 +125,7 @@ class GeologicalModel:
         self.feature_name_index = {}
         self._data = pd.DataFrame()  # None
 
-        self.stratigraphic_column = StratigraphicColumn()
-
+        self._stratigraphic_column = StratigraphicColumn()
 
         self.tol = 1e-10 * np.max(self.bounding_box.maximum - self.bounding_box.origin)
         self._dtm = None
@@ -187,7 +188,6 @@ class GeologicalModel:
         ].astype(float)
         return data
 
-
         if "type" in data:
             logger.warning("'type' is deprecated replace with 'feature_name' \n")
             data.rename(columns={"type": "feature_name"}, inplace=True)
@@ -332,7 +332,21 @@ class GeologicalModel:
             name of the feature to return
         """
         return self.get_feature_by_name(feature_name)
+    def __setitem__(self, feature_name, feature):
+        """Set a feature in the model using feature_name_index
 
+        Parameters
+        ----------
+        feature_name : string
+            name of the feature to set
+        feature : GeologicalFeature
+            the geological feature to set
+        """
+        if not issubclass(type(feature), BaseFeature):
+            raise TypeError("feature must be a GeologicalFeature")
+        if feature.name != feature_name:
+            raise ValueError("feature name does not match key")
+        self._add_feature(feature)
     def __contains__(self, feature_name):
         return feature_name in self.feature_name_index
 
@@ -409,7 +423,6 @@ class GeologicalModel:
         """
         return [f.name for f in self.faults]
 
-
     def to_file(self, file):
         """Save a model to a pickle file requires dill
 
@@ -429,7 +442,7 @@ class GeologicalModel:
         except pickle.PicklingError:
             logger.error("Error saving file")
 
-    def _add_feature(self, feature):
+    def _add_feature(self, feature, index: Optional[int] = None):
         """
         Add a feature to the model stack
 
@@ -446,9 +459,18 @@ class GeologicalModel:
             )
             self.features[self.feature_name_index[feature.name]] = feature
         else:
-            self.features.append(feature)
-            self.feature_name_index[feature.name] = len(self.features) - 1
-            logger.info(f"Adding {feature.name} to model at location {len(self.features)}")
+            if index is not None:
+                if index < 0 or index > len(self.features):
+                    raise IndexError(f"Index {index} out of bounds for features list")
+                self.features.insert(index, feature)
+                self.feature_name_index[feature.name] = index
+                logger.info(f"Adding {feature.name} to model at location {index}")
+                for index, feature in enumerate(self.features):
+                    self.feature_name_index[feature.name] = index
+            else:
+                self.features.append(feature)
+                self.feature_name_index[feature.name] = len(self.features) - 1
+                logger.info(f"Adding {feature.name} to model at location {len(self.features)}")
         self._add_domain_fault_above(feature)
         if feature.type == FeatureType.INTERPOLATED:
             self._add_unconformity_above(feature)
@@ -506,10 +528,34 @@ class GeologicalModel:
         self._data = data.copy()
         # self._data[['X','Y','Z']] = self.bounding_box.project(self._data[['X','Y','Z']].to_numpy())
 
-
     def set_model_data(self, data):
         logger.warning("deprecated method. Model data can now be set using the data attribute")
         self.data = data.copy()
+    @property
+    def stratigraphic_column(self):
+        """Get the stratigraphic column of the model
+
+        Returns
+        -------
+        StratigraphicColumn
+            the stratigraphic column of the model
+        """
+        return self._stratigraphic_column
+    @stratigraphic_column.setter
+    def stratigraphic_column(self, stratigraphic_column: Union[StratigraphicColumn,Dict]):
+        """Set the stratigraphic column of the model
+
+        Parameters
+        ----------
+        stratigraphic_column : StratigraphicColumn
+            the stratigraphic column to set
+        """
+        if isinstance(stratigraphic_column, dict):
+            self.set_stratigraphic_column(stratigraphic_column)
+            return
+        elif not isinstance(stratigraphic_column, StratigraphicColumn):
+            raise ValueError("stratigraphic_column must be a StratigraphicColumn object")
+        self._stratigraphic_column = stratigraphic_column
 
     def set_stratigraphic_column(self, stratigraphic_column, cmap="tab20"):
         """
@@ -532,21 +578,24 @@ class GeologicalModel:
         }
 
         """
-        self.stratigraphic_column.clear()
+        self.stratigraphic_column.clear(basement=False)
         # if the colour for a unit hasn't been specified we can just sample from
         # a colour map e.g. tab20
         logger.info("Adding stratigraphic column to model")
         DeprecationWarning(
             "set_stratigraphic_column is deprecated, use model.stratigraphic_column.add_units instead"
         )
-        for g in stratigraphic_column.keys():
+        for i, g in enumerate(stratigraphic_column.keys()):
+            if g == 'faults':
+                logger.info('Not adding faults to stratigraphic column')
+                continue
             for u in stratigraphic_column[g].keys():
                 thickness = 0
                 if "min" in stratigraphic_column[g][u] and "max" in stratigraphic_column[g][u]:
                     min_val = stratigraphic_column[g][u]["min"]
                     max_val = stratigraphic_column[g][u].get("max", None)
                     thickness = max_val - min_val if max_val is not None else None
-                logger.warning(
+                logger.info(
                     f"""
                                model.stratigraphic_column.add_unit({u},
                                colour={stratigraphic_column[g][u].get("colour", None)},
@@ -557,17 +606,20 @@ class GeologicalModel:
                     colour=stratigraphic_column[g][u].get("colour", None),
                     thickness=thickness,
                 )
+
             self.stratigraphic_column.add_unconformity(
                 name=''.join([g, 'unconformity']),
             )
+            self.stratigraphic_column.group_mapping[f'Group_{i}'] = g
 
     def create_and_add_foliation(
         self,
         series_surface_name: str,
         *,
-        series_surface_data: pd.DataFrame = None,
+        index: Optional[int] = None,
+        data: Optional[pd.DataFrame] = None,
         interpolatortype: str = "FDI",
-        nelements: int = 1000,
+        nelements: int = LoopStructuralConfig.nelements,
         tol=None,
         faults=None,
         **kwargs,
@@ -620,13 +672,13 @@ class GeologicalModel:
             **kwargs,
         )
         # add data
-        if series_surface_data is None:
-            series_surface_data = self.data.loc[self.data["feature_name"] == series_surface_name]
+        if data is None:
+            data = self.data.loc[self.data["feature_name"] == series_surface_name]
 
-        if series_surface_data.shape[0] == 0:
+        if data.shape[0] == 0:
             logger.warning("No data for {series_surface_data}, skipping")
             return
-        series_builder.add_data_from_data_frame(self.prepare_data(series_surface_data))
+        series_builder.add_data_from_data_frame(self.prepare_data(data))
         self._add_faults(series_builder, features=faults)
 
         # build feature
@@ -637,16 +689,17 @@ class GeologicalModel:
         # could just pass a regular grid of points - mask by any above unconformities??
 
         series_feature.type = FeatureType.INTERPOLATED
-        self._add_feature(series_feature)
+        self._add_feature(series_feature,index=index)
         return series_feature
 
     def create_and_add_fold_frame(
         self,
         fold_frame_name: str,
         *,
-        fold_frame_data=None,
+        index: Optional[int] = None,
+        data=None,
         interpolatortype="FDI",
-        nelements=1000,
+        nelements=LoopStructuralConfig.nelements,
         tol=None,
         buffer=0.1,
         **kwargs,
@@ -694,12 +747,12 @@ class GeologicalModel:
             **kwargs,
         )
         # add data
-        if fold_frame_data is None:
-            fold_frame_data = self.data.loc[self.data["feature_name"] == fold_frame_name]
-        if fold_frame_data.shape[0] == 0:
+        if data is None:
+            data = self.data.loc[self.data["feature_name"] == fold_frame_name]
+        if data.shape[0] == 0:
             logger.warning(f"No data for {fold_frame_name}, skipping")
             return
-        fold_frame_builder.add_data_from_data_frame(self.prepare_data(fold_frame_data))
+        fold_frame_builder.add_data_from_data_frame(self.prepare_data(data))
         self._add_faults(fold_frame_builder[0])
         self._add_faults(fold_frame_builder[1])
         self._add_faults(fold_frame_builder[2])
@@ -709,7 +762,7 @@ class GeologicalModel:
 
         fold_frame.type = FeatureType.STRUCTURALFRAME
         fold_frame.builder = fold_frame_builder
-        self._add_feature(fold_frame)
+        self._add_feature(fold_frame,index=index)
 
         return fold_frame
 
@@ -717,9 +770,10 @@ class GeologicalModel:
         self,
         foliation_name,
         *,
-        foliation_data=None,
+        index: Optional[int] = None,
+        data=None,
         interpolatortype="DFI",
-        nelements=10000,
+        nelements=LoopStructuralConfig.nelements,
         buffer=0.1,
         fold_frame=None,
         svario=True,
@@ -763,7 +817,7 @@ class GeologicalModel:
             fold_frame = self.features[-1]
         assert isinstance(fold_frame, FoldFrame), "Please specify a FoldFrame"
 
-        fold = FoldEvent(fold_frame, name=f"Fold_{foliation_data}", invert_norm=invert_fold_norm)
+        fold = FoldEvent(fold_frame, name=f"Fold_{foliation_name}", invert_norm=invert_fold_norm)
 
         if interpolatortype != "DFI":
             logger.warning("Folded foliation only supports DFI interpolator, changing to DFI")
@@ -778,12 +832,12 @@ class GeologicalModel:
             model=self,
             **kwargs,
         )
-        if foliation_data is None:
-            foliation_data = self.data.loc[self.data["feature_name"] == foliation_name]
-        if foliation_data.shape[0] == 0:
+        if data is None:
+            data = self.data.loc[self.data["feature_name"] == foliation_name]
+        if data.shape[0] == 0:
             logger.warning(f"No data for {foliation_name}, skipping")
             return
-        series_builder.add_data_from_data_frame(self.prepare_data(foliation_data))
+        series_builder.add_data_from_data_frame(self.prepare_data(data))
 
         self._add_faults(series_builder)
         # series_builder.add_data_to_interpolator(True)
@@ -794,19 +848,20 @@ class GeologicalModel:
         # series_feature = series_builder.build(**kwargs)
         series_feature = series_builder.feature
         series_builder.update_build_arguments(kwargs)
-        series_feature.type = FeatureType.INTERPOLATED
+        series_feature.type = FeatureType.FOLDED
         series_feature.fold = fold
 
-        self._add_feature(series_feature)
+        self._add_feature(series_feature,index)
         return series_feature
 
     def create_and_add_folded_fold_frame(
         self,
         fold_frame_name: str,
         *,
-        fold_frame_data: Optional[pd.DataFrame] = None,
+        index: Optional[int] = None,
+        data: Optional[pd.DataFrame] = None,
         interpolatortype="FDI",
-        nelements=10000,
+        nelements=LoopStructuralConfig.nelements,
         fold_frame=None,
         tol=None,
         **kwargs,
@@ -859,7 +914,7 @@ class GeologicalModel:
             logger.info("Using last feature as fold frame")
             fold_frame = self.features[-1]
         assert isinstance(fold_frame, FoldFrame), "Please specify a FoldFrame"
-        fold = FoldEvent(fold_frame, name=f"Fold_{fold_frame_data}")
+        fold = FoldEvent(fold_frame, name=f"Fold_{fold_frame_name}")
 
         interpolatortypes = [
             "DFI",
@@ -876,9 +931,9 @@ class GeologicalModel:
             model=self,
             **kwargs,
         )
-        if fold_frame_data is None:
-            fold_frame_data = self.data[self.data["feature_name"] == fold_frame_name]
-        fold_frame_builder.add_data_from_data_frame(self.prepare_data(fold_frame_data))
+        if data is None:
+            data = self.data[self.data["feature_name"] == fold_frame_name]
+        fold_frame_builder.add_data_from_data_frame(self.prepare_data(data))
 
         for i in range(3):
             self._add_faults(fold_frame_builder[i])
@@ -892,7 +947,7 @@ class GeologicalModel:
 
         folded_fold_frame.type = FeatureType.STRUCTURALFRAME
 
-        self._add_feature(folded_fold_frame)
+        self._add_feature(folded_fold_frame,index=index)
 
         return folded_fold_frame
 
@@ -955,14 +1010,14 @@ class GeologicalModel:
 
         interpolatortype = kwargs.get("interpolatortype", "PLI")
         # buffer = kwargs.get("buffer", 0.1)
-        nelements = kwargs.get("nelements", 1e2)
+        nelements = kwargs.get("nelements", LoopStructuralConfig.nelements)
 
         weights = [gxxgz, gxxgy, gyxgz]
 
         intrusion_frame_builder = IntrusionFrameBuilder(
             interpolatortype=interpolatortype,
             bounding_box=self.bounding_box.with_buffer(kwargs.get("buffer", 0.1)),
-            nelements=kwargs.get("nelements", 1e2),
+            nelements=kwargs.get("nelements", LoopStructuralConfig.nelements),
             name=intrusion_frame_name,
             model=self,
             **kwargs,
@@ -1105,7 +1160,7 @@ class GeologicalModel:
                 feature.add_region(f)
                 break
 
-    def add_unconformity(self, feature: GeologicalFeature, value: float) -> UnconformityFeature:
+    def add_unconformity(self, feature: GeologicalFeature, value: float, index: Optional[int] = None) -> UnconformityFeature:
         """
         Use an existing feature to add an unconformity to the model.
 
@@ -1142,10 +1197,10 @@ class GeologicalModel:
             else:
                 f.add_region(uc_feature)
         # now add the unconformity to the feature list
-        self._add_feature(uc_feature)
+        self._add_feature(uc_feature,index=index)
         return uc_feature
 
-    def add_onlap_unconformity(self, feature: GeologicalFeature, value: float) -> GeologicalFeature:
+    def add_onlap_unconformity(self, feature: GeologicalFeature, value: float, index: Optional[int] = None) -> GeologicalFeature:
         """
         Use an existing feature to add an unconformity to the model.
 
@@ -1173,12 +1228,18 @@ class GeologicalModel:
                 continue
             if f != feature:
                 f.add_region(uc_feature)
-        self._add_feature(uc_feature.inverse())
+        self._add_feature(uc_feature.inverse(),index=index)
 
         return uc_feature
 
     def create_and_add_domain_fault(
-        self, fault_surface_data, *, nelements=10000, interpolatortype="FDI", **kwargs
+        self,
+        fault_surface_data,
+        *,
+        nelements=LoopStructuralConfig.nelements,
+        interpolatortype="FDI",
+        index: Optional[int] = None,
+        **kwargs,
     ):
         """
         Parameters
@@ -1219,7 +1280,7 @@ class GeologicalModel:
         domain_fault = domain_fault_feature_builder.feature
         domain_fault_feature_builder.update_build_arguments(kwargs)
         domain_fault.type = FeatureType.DOMAINFAULT
-        self._add_feature(domain_fault)
+        self._add_feature(domain_fault, index=index)
         self._add_domain_fault_below(domain_fault)
 
         domain_fault_uc = UnconformityFeature(domain_fault, 0)
@@ -1232,7 +1293,8 @@ class GeologicalModel:
         fault_name: str,
         displacement: float,
         *,
-        fault_data: Optional[pd.DataFrame] = None,
+        index: Optional[int] = None,
+        data: Optional[pd.DataFrame] = None,
         interpolatortype="FDI",
         tol=None,
         fault_slip_vector=None,
@@ -1321,14 +1383,14 @@ class GeologicalModel:
         fault_frame_builder = FaultBuilder(
             interpolatortype,
             bounding_box=self.bounding_box,
-            nelements=kwargs.pop("nelements", 1e4),
+            nelements=kwargs.pop("nelements", LoopStructuralConfig.nelements),
             name=fault_name,
             model=self,
             **kwargs,
         )
-        if fault_data is None:
-            fault_data = self.data.loc[self.data["feature_name"] == fault_name]
-        if fault_data.shape[0] == 0:
+        if data is None:
+            data = self.data.loc[self.data["feature_name"] == fault_name]
+        if data.shape[0] == 0:
             logger.warning(f"No data for {fault_name}, skipping")
             return
 
@@ -1344,7 +1406,7 @@ class GeologicalModel:
         if intermediate_axis:
             intermediate_axis = intermediate_axis
         fault_frame_builder.create_data_from_geometry(
-            fault_frame_data=self.prepare_data(fault_data),
+            fault_frame_data=self.prepare_data(data),
             fault_center=fault_center,
             fault_normal_vector=fault_normal_vector,
             fault_slip_vector=fault_slip_vector,
@@ -1376,7 +1438,7 @@ class GeologicalModel:
                 break
         if displacement == 0:
             fault.type = FeatureType.INACTIVEFAULT
-        self._add_feature(fault)
+        self._add_feature(fault,index=index)
 
         return fault
 
@@ -1400,7 +1462,6 @@ class GeologicalModel:
 
         return self.bounding_box.reproject(points, inplace=inplace)
 
-
     # TODO move scale to bounding box/transformer
     def scale(self, points: np.ndarray, *, inplace: bool = False) -> np.ndarray:
         """Take points in UTM coordinates and reproject
@@ -1419,7 +1480,6 @@ class GeologicalModel:
         """
         return self.bounding_box.project(np.array(points).astype(float), inplace=inplace)
 
-
     def regular_grid(self, *, nsteps=None, shuffle=True, rescale=False, order="C"):
         """
         Return a regular grid within the model bounding box
@@ -1494,22 +1554,18 @@ class GeologicalModel:
         if self.stratigraphic_column is None:
             logger.warning("No stratigraphic column defined")
             return strat_id
-        for group in reversed(self.stratigraphic_column.keys()):
-            if group == "faults":
-                continue
-            feature_id = self.feature_name_index.get(group, -1)
+
+        s_id = 0
+        for g in reversed(self.stratigraphic_column.get_groups()):
+            feature_id = self.feature_name_index.get(g.name, -1)
             if feature_id >= 0:
-                feature = self.features[feature_id]
-                vals = feature.evaluate_value(xyz)
-                for series in self.stratigraphic_column[group].values():
-                    strat_id[
-                        np.logical_and(
-                            vals < series.get("max", feature.max()),
-                            vals > series.get("min", feature.min()),
-                        )
-                    ] = series["id"]
+                vals = self.features[feature_id].evaluate_value(xyz)
+                for u in g.units:
+                    strat_id[np.logical_and(vals < u.max(), vals > u.min())] = s_id
+                    s_id += 1
             if feature_id == -1:
-                logger.error(f"Model does not contain {group}")
+                logger.error(f"Model does not contain {g.name}")
+
         return strat_id
 
     def evaluate_model_gradient(self, points: np.ndarray, *, scale: bool = True) -> np.ndarray:
@@ -1531,16 +1587,13 @@ class GeologicalModel:
         if scale:
             xyz = self.scale(xyz, inplace=False)
         grad = np.zeros(xyz.shape)
-        for group in reversed(self.stratigraphic_column.keys()):
-            if group == "faults":
-                continue
-            feature_id = self.feature_name_index.get(group, -1)
+        for g in reversed(self.stratigraphic_column.get_groups()):
+            feature_id = self.feature_name_index.get(g.name, -1)
             if feature_id >= 0:
-                feature = self.features[feature_id]
-                gradt = feature.evaluate_gradient(xyz)
+                gradt = self.features[feature_id].evaluate_gradient(xyz)
                 grad[~np.isnan(gradt).any(axis=1)] = gradt[~np.isnan(gradt).any(axis=1)]
             if feature_id == -1:
-                logger.error(f"Model does not contain {group}")
+                logger.error(f"Model does not contain {g.name}")
 
         return grad
 
@@ -1710,16 +1763,7 @@ class GeologicalModel:
         ids : list
             list of unique stratigraphic ids, featurename, unit name and min and max scalar values
         """
-        ids = []
-        if self.stratigraphic_column is None:
-            logger.warning('No stratigraphic column defined')
-            return ids
-        for group in self.stratigraphic_column.keys():
-            if group == "faults":
-                continue
-            for name, series in self.stratigraphic_column[group].items():
-                ids.append([series["id"], group, name, series['min'], series['max']])
-        return ids
+        return self.stratigraphic_column.get_stratigraphic_ids()
 
     def get_fault_surfaces(self, faults: List[str] = []):
         surfaces = []
