@@ -1,8 +1,7 @@
 import enum
-from typing import Dict
+from typing import Dict, Optional, List, Tuple
 import numpy as np
-from LoopStructural.utils import rng, getLogger
-
+from LoopStructural.utils import rng, getLogger, Observable, random_colour
 logger = getLogger(__name__)
 logger.info("Imported LoopStructural Stratigraphic Column module")
 class UnconformityType(enum.Enum):
@@ -40,24 +39,59 @@ class StratigraphicColumnElement:
         self.uuid = uuid
 
 
-class StratigraphicUnit(StratigraphicColumnElement):
+class StratigraphicUnit(StratigraphicColumnElement, Observable['StratigraphicUnit']):
     """
     A class to represent a stratigraphic unit.
     """
 
-    def __init__(self, *, uuid=None, name=None, colour=None, thickness=None, data=None):
+    def __init__(self, *, uuid=None, name=None, colour=None, thickness=None, data=None, id=None):
         """
         Initializes the StratigraphicUnit with a name and an optional description.
         """
-        super().__init__(uuid)
+        StratigraphicColumnElement.__init__(self, uuid)
+        Observable.__init__(self)
         self.name = name
         if colour is None:
             colour = rng.random(3)
         self.colour = colour
-        self.thickness = thickness
+        self._thickness = thickness
         self.data = data
         self.element_type = StratigraphicColumnElementType.UNIT
-
+        self.id = id
+        self.min_value = None  # Minimum scalar field value for the unit
+        self.max_value = None  # Maximum scalar field value for the unit
+    @property
+    def id(self):
+        return self._id
+    @property
+    def thickness(self):
+        return self._thickness
+    @thickness.setter
+    def thickness(self, value):
+        """
+        Sets the thickness of the unit.
+        """
+        self._thickness = value
+        self.notify('unit/thickness_updated', unit=self)
+    @id.setter
+    def id(self, value):
+        """
+        Sets the ID of the unit.
+        """
+        if not isinstance(value, int):
+            raise TypeError("ID must be an integer")
+        self._id = value
+        self.notify('unit/id_updated', unit=self)
+    def min(self):
+        """
+        Returns the minimum value of the unit.
+        """
+        return self.min_value if self.min_value is not None else 0
+    def max(self):
+        """
+        Returns the maximum value of the unit.
+        """
+        return self.max_value if self.max_value is not None else np.inf
     def to_dict(self):
         """
         Converts the stratigraphic unit to a dictionary representation.
@@ -65,7 +99,7 @@ class StratigraphicUnit(StratigraphicColumnElement):
         colour = self.colour
         if isinstance(colour, np.ndarray):
             colour = colour.astype(float).tolist()
-        return {"name": self.name, "colour": colour, "thickness": self.thickness, 'uuid': self.uuid}
+        return {"name": self.name, "colour": colour, "thickness": self.thickness, 'uuid': self.uuid, 'id': self.id}
 
     @classmethod
     def from_dict(cls, data):
@@ -78,7 +112,7 @@ class StratigraphicUnit(StratigraphicColumnElement):
         colour = data.get("colour")
         thickness = data.get("thickness", None)
         uuid = data.get("uuid", None)
-        return cls(uuid=uuid, name=name, colour=colour, thickness=thickness)
+        return cls(uuid=uuid, name=name, colour=colour, thickness=thickness, id=data.get("id", None))
 
     def __str__(self):
         """
@@ -154,7 +188,7 @@ class StratigraphicGroup:
         self.units = units if units is not None else []
 
 
-class StratigraphicColumn:
+class StratigraphicColumn(Observable['StratigraphicColumn']):
     """
     A class to represent a stratigraphic column, which is a vertical section of the Earth's crust
     showing the sequence of rock layers and their relationships.
@@ -164,20 +198,38 @@ class StratigraphicColumn:
         """
         Initializes the StratigraphicColumn with a name and a list of layers.
         """
-        self.order = [StratigraphicUnit(name='Basement', colour='grey', thickness=np.inf),StratigraphicUnconformity(name='Base Unconformity', unconformity_type=UnconformityType.ERODE)]
+        super().__init__()
+        self.order = []
+        self.add_basement()
         self.group_mapping = {}
-    def clear(self,basement=True):
+
+    def get_new_id(self):
+        """
+        Generates a new unique ID for a stratigraphic unit.
+        """
+        if not self.order:
+            return 0
+        return max([u.id for u in self.order if isinstance(u, StratigraphicUnit)], default=0) + 1
+    def add_basement(self):
+        self.add_unit(name='Basement', colour='grey', thickness=np.inf)
+        self.add_unconformity(
+            name='Base Unconformity', unconformity_type=UnconformityType.ERODE
+        )
+    def clear(self, basement=True):
         """
         Clears the stratigraphic column, removing all elements.
         """
         if basement:
-            self.order = [StratigraphicUnit(name='Basement', colour='grey', thickness=np.inf),StratigraphicUnconformity(name='Base Unconformity', unconformity_type=UnconformityType.ERODE)]
+            self.add_basement()
+            
         else:
             self.order = []
         self.group_mapping = {}
-
-    def add_unit(self, name,*, colour=None, thickness=None, where='top'):
-        unit = StratigraphicUnit(name=name, colour=colour, thickness=thickness)
+        self.notify('column_cleared')
+    def add_unit(self, name,*, colour=None, thickness=None, where='top',id=None):
+        if id is None:
+            id = self.get_new_id()
+        unit = StratigraphicUnit(name=name, colour=colour, thickness=thickness, id=id)
 
         if where == 'top':
             self.order.append(unit)
@@ -185,7 +237,9 @@ class StratigraphicColumn:
             self.order.insert(0, unit)
         else:
             raise ValueError("Invalid 'where' argument. Use 'top' or 'bottom'.")
-
+        unit.attach(self.update_unit_values,'unit/*')
+        self.notify('unit_added', unit=unit)
+        self.update_unit_values()  # Update min and max values after adding a unit
         return unit
 
     def remove_unit(self, uuid):
@@ -195,7 +249,9 @@ class StratigraphicColumn:
         for i, element in enumerate(self.order):
             if element.uuid == uuid:
                 del self.order[i]
+                self.notify('unit_removed', uuid=uuid)
                 return True
+
         return False
 
     def add_unconformity(self, name, *, unconformity_type=UnconformityType.ERODE, where='top' ):
@@ -209,6 +265,7 @@ class StratigraphicColumn:
             self.order.insert(0, unconformity)
         else:
             raise ValueError("Invalid 'where' argument. Use 'top' or 'bottom'.")
+        self.notify('unconformity_added', unconformity=unconformity)
         return unconformity
 
     def get_element_by_index(self, index):
@@ -228,6 +285,7 @@ class StratigraphicColumn:
                 return unit
 
         return None
+
     def get_unconformity_by_name(self, name):
         """
         Retrieves an unconformity by its name from the stratigraphic column.
@@ -245,6 +303,15 @@ class StratigraphicColumn:
             if element.uuid == uuid:
                 return element
         raise KeyError(f"No element found with uuid: {uuid}")
+
+    def get_group_for_unit_name(self, unit_name:str) -> Optional[StratigraphicGroup]:
+        """
+        Retrieves the group for a given unit name.
+        """
+        for group in self.get_groups():
+            if any(unit.name == unit_name for unit in group.units):
+                return group
+        return None
     def add_element(self, element):
         """
         Adds a StratigraphicColumnElement to the stratigraphic column.
@@ -287,7 +354,15 @@ class StratigraphicColumn:
         if group:
             groups.append(group)
         return groups
+    def get_stratigraphic_ids(self) -> List[List[str]]:
+        ids = []
+        for group in self.get_groups():
+            if group == "faults":
+                continue
 
+            for unit in group.units:
+                ids.append([unit.id, group, unit.name, unit.min(), unit.max()])
+        return ids
     def get_unitname_groups(self):
         groups = self.get_groups()
         groups_list = []
@@ -296,7 +371,18 @@ class StratigraphicColumn:
             group = [u.name for u in g.units if isinstance(u, StratigraphicUnit)]
             groups_list.append(group)
         return groups_list
-        
+
+    def get_group_unit_pairs(self) -> List[Tuple[str,str]]:
+        """
+        Returns a list of tuples containing group names and unit names.
+        """
+        groups = self.get_groups()
+        group_unit_pairs = []
+        for g in groups:
+            for u in g.units:
+                if isinstance(u, StratigraphicUnit):
+                    group_unit_pairs.append((g.name, u.name))
+        return group_unit_pairs
 
     def __getitem__(self, uuid):
         """
@@ -316,6 +402,21 @@ class StratigraphicColumn:
         self.order = [
             self.__getitem__(uuid) for uuid in new_order if self.__getitem__(uuid) is not None
         ]
+        self.notify('order_updated', new_order=self.order)
+        self.update_unit_values()  # Update min and max values after updating the order
+    def update_unit_values(self, *, observable: Optional["Observable"] = None, event: Optional[str]= None):
+        """
+        Updates the min and max values for each unit based on their position in the column.
+        """
+        # If the event is not 'unit/*', skip the update
+        if event is not None and event != 'unit/*':
+            return
+        cumulative_thickness = 0
+        for element in self.order:
+            if isinstance(element, StratigraphicUnit):
+                element.min_value = cumulative_thickness
+                element.max_value = cumulative_thickness + (element.thickness or 0)
+                cumulative_thickness = element.max_value
 
     def update_element(self, unit_data: Dict):
         """
@@ -334,6 +435,8 @@ class StratigraphicColumn:
             element.unconformity_type = UnconformityType(
                 unit_data.get('unconformity_type', element.unconformity_type.value)
             )
+        self.notify('element_updated', element=element)
+        self.update_unit_values()  # Update min and max values after updating an element
 
     def __str__(self):
         """
@@ -354,14 +457,15 @@ class StratigraphicColumn:
         """
         if not isinstance(data, dict):
             raise TypeError("Data must be a dictionary")
-        self.clear(basement=False)
-        elements_data = data.get("elements", [])
-        for element_data in elements_data:
-            if "unconformity_type" in element_data:
-                element = StratigraphicUnconformity.from_dict(element_data)
-            else:
-                element = StratigraphicUnit.from_dict(element_data)
-            self.add_element(element)
+        with self.freeze_notifications():
+            self.clear(basement=False)
+            elements_data = data.get("elements", [])
+            for element_data in elements_data:
+                if "unconformity_type" in element_data:
+                    element = StratigraphicUnconformity.from_dict(element_data)
+                else:
+                    element = StratigraphicUnit.from_dict(element_data)
+                self.add_element(element)
     @classmethod
     def from_dict(cls, data):
         """
@@ -471,3 +575,29 @@ class StratigraphicColumn:
         ax.axis("off")
 
         return fig
+    
+    def cmap(self):
+        try:
+            import matplotlib.colors as colors
+
+            colours = []
+            boundaries = []
+            data = []
+            for group in self.get_groups():
+                for u in group.units:
+                    colour = u.colour
+                    if not isinstance(colour, str):
+                        try:
+                            u.colour = colors.to_hex(colour)
+                        except ValueError:
+                            logger.warning(f"Cannot convert colour {colour} to hex, using default")
+                            u.colour = random_colour()
+                    data.append((u.id, u.colour))
+                    colours.append(u.colour)
+                    boundaries.append(u.id)
+             # print(u,v)
+            cmap = colors.ListedColormap(colours)
+        except ImportError:
+            logger.warning("Cannot use predefined colours as I can't import matplotlib")
+            cmap = "tab20"
+        return cmap
\ No newline at end of file
