@@ -1,3 +1,3 @@
-from ._fault_function import Composite, CubicFunction, Ones, Zeros
+from ._fault_function import Composite, CubicFunction, Ones, Zeros, FaultDisplacement
 from ._fault_function_feature import FaultDisplacementFeature
 from ._fault_segment import FaultSegment
