@@ -115,12 +115,17 @@ class LoopIsosurfacer:
                 values,
             )
         logger.info(f'Isosurfacing at values: {isovalues}')
+        individual_names = False
         if name is None:
             names = ["surface"] * len(isovalues)
         if isinstance(name, str):
             names = [name] * len(isovalues)
+            if len(isovalues) == 1:
+                individual_names = True
         if isinstance(name, list):
             names = name
+            if len(names) == len(isovalues):
+                individual_names = True
         if colours is None:
             colours = [None] * len(isovalues)
         for name, isovalue, colour in zip(names, isovalues, colours):
@@ -151,7 +156,7 @@ class LoopIsosurfacer:
                     vertices=verts,
                     triangles=faces,
                     normals=normals,
-                    name=f"{name}_{isovalue}",
+                    name=name if individual_names else f"{name}_{isovalue}",
                     values=values,
                     colour=colour,
                 )
